@@ -1,9 +1,11 @@
 """
-Copyright 2019 by J. Christopher Wagner (jwag). All rights reserved.
+Copyright 2019-2021 by J. Christopher Wagner (jwag). All rights reserved.
 :license: MIT, see LICENSE for more details.
 
 
-Complete models for all features when using Flask-SqlAlchemy
+Complete models for all features when using Flask-SqlAlchemy.
+
+You can change the table names by passing them in to the set_db_info() method.
 
 BE AWARE: Once any version of this is shipped no changes can be made - instead
 a new version needs to be created.
@@ -26,7 +28,7 @@ from sqlalchemy.sql import func
 from flask_security import RoleMixin, UserMixin
 
 
-class FsModels(object):
+class FsModels:
     """
     Helper class for model mixins.
     This records the ``db`` (which is a Flask-SqlAlchemy object) for use in
@@ -36,18 +38,22 @@ class FsModels(object):
     roles_users = None
     db = None
     fs_model_version = 1
+    user_table_name = "user"
+    role_table_name = "role"
 
     @classmethod
-    def set_db_info(cls, appdb):
-        """ Initialize Model.
+    def set_db_info(cls, appdb, user_table_name="user", role_table_name="role"):
+        """Initialize Model.
         This needs to be called after the DB object has been created
         (e.g. db = Sqlalchemy())
         """
         cls.db = appdb
+        cls.user_table_name = user_table_name
+        cls.role_table_name = role_table_name
         cls.roles_users = appdb.Table(
             "roles_users",
-            Column("user_id", Integer(), ForeignKey("user.id")),
-            Column("role_id", Integer(), ForeignKey("role.id")),
+            Column("user_id", Integer(), ForeignKey(f"{cls.user_table_name}.id")),
+            Column("role_id", Integer(), ForeignKey(f"{cls.role_table_name}.id")),
         )
 
 
@@ -56,9 +62,9 @@ class FsRoleMixin(RoleMixin):
     name = Column(String(80), unique=True, nullable=False)
     description = Column(String(255))
     # A comma separated list of strings
-    permissions = Column(UnicodeText, nullable=True)
+    permissions = Column(UnicodeText, nullable=True)  # type: ignore
     update_datetime = Column(
-        DateTime,
+        type_=DateTime,
         nullable=False,
         server_default=func.now(),
         onupdate=datetime.datetime.utcnow,
@@ -66,8 +72,7 @@ class FsRoleMixin(RoleMixin):
 
 
 class FsUserMixin(UserMixin):
-    """ User information
-    """
+    """User information"""
 
     # flask_security basic fields
     id = Column(Integer, primary_key=True)
@@ -77,7 +82,7 @@ class FsUserMixin(UserMixin):
     password = Column(String(255), nullable=False)
     active = Column(Boolean(), nullable=False)
 
-    # Faster token checking
+    # Flask-Security user identifier
     fs_uniquifier = Column(String(64), unique=True, nullable=False)
 
     # confirmable
@@ -97,15 +102,16 @@ class FsUserMixin(UserMixin):
 
     @declared_attr
     def roles(cls):
+        # The first arg is a class name, the backref is a column name
         return FsModels.db.relationship(
             "Role",
             secondary=FsModels.roles_users,
             backref=FsModels.db.backref("users", lazy="dynamic"),
         )
 
-    create_datetime = Column(DateTime, nullable=False, server_default=func.now())
+    create_datetime = Column(type_=DateTime, nullable=False, server_default=func.now())
     update_datetime = Column(
-        DateTime,
+        type_=DateTime,
         nullable=False,
         server_default=func.now(),
         onupdate=datetime.datetime.utcnow,
@@ -117,8 +123,8 @@ These are placeholders - not current used
 """
 
 
-class FsOauth2ClientMixin(object):
-    """ Oauth2 client """
+class FsOauth2ClientMixin:
+    """Oauth2 client"""
 
     id = Column(String(64), primary_key=True)
 
@@ -138,8 +144,8 @@ class FsOauth2ClientMixin(object):
     redirect_uris = Column(UnicodeText())
 
 
-class FsTokenMixin(object):
-    """ (Bearer) Tokens that have been given out """
+class FsTokenMixin:
+    """(Bearer) Tokens that have been given out"""
 
     id = Column(Integer, primary_key=True)
 
@@ -160,5 +166,5 @@ class FsTokenMixin(object):
     revoked = Column(Boolean(), nullable=False, default=False)
     access_token = Column(String(100), unique=True, nullable=False)
     refresh_token = Column(String(100), unique=True)
-    issued_at = Column(DateTime, nullable=False, server_default=func.now())
+    issued_at = Column(type_=DateTime, nullable=False, server_default=func.now())
     expires_at = Column(DateTime())
