@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     flask_security.views
     ~~~~~~~~~~~~~~~~~~~~
@@ -6,7 +5,7 @@
     Flask-Security views module
 
     :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2020 by J. Christopher Wagner (jwag).
+    :copyright: (c) 2019-2021 by J. Christopher Wagner (jwag).
     :license: MIT, see LICENSE for more details.
 
     CSRF is tricky. By default all our forms have CSRF protection built in via
@@ -27,25 +26,21 @@
     so app must set CSRF_IGNORE_UNAUTH_ENDPOINTS (or use CSRF/session cookie for logging
     in then once they have a token, no need for cookie).
 
-    TODO: two-factor routes such as tf_setup need work. They seem to support both
-    authenticated (via session?) as well as unauthenticated access.
 """
 
-import sys
+from functools import partial
 import time
+import typing as t
 
 from flask import (
     Blueprint,
-    abort,
     after_this_request,
-    current_app,
     jsonify,
     request,
     session,
 )
 from flask_login import current_user
-from werkzeug.datastructures import MultiDict
-from werkzeug.local import LocalProxy
+from werkzeug.datastructures import ImmutableMultiDict, MultiDict
 
 from .changeable import change_user_password
 from .confirmable import (
@@ -55,11 +50,11 @@ from .confirmable import (
 )
 from .decorators import anonymous_user_required, auth_required, unauth_csrf
 from .passwordless import login_token_status, send_login_instructions
+from .proxies import _security, _datastore
 from .quart_compat import get_quart_status
 from .unified_signin import (
     us_signin,
     us_signin_send_code,
-    us_qrcode,
     us_setup,
     us_setup_validate,
     us_verify,
@@ -74,26 +69,34 @@ from .recoverable import (
 from .registerable import register_user
 from .twofactor import (
     complete_two_factor_process,
+    generate_tf_validity_token,
+    is_tf_setup,
     tf_clean_session,
     tf_disable,
     tf_login,
+    tf_set_validity_token_cookie,
+    tf_verify_validility_token,
 )
 from .utils import (
     base_render_json,
-    config_value,
+    check_and_update_authn_fresh,
+    config_value as cv,
     do_flash,
     get_message,
     get_post_login_redirect,
     get_post_logout_redirect,
     get_post_register_redirect,
     get_post_verify_redirect,
+    get_request_attr,
     get_url,
     json_error_response,
     login_user,
     logout_user,
+    send_mail,
     slash_url_suffix,
     suppress_form_csrf,
     url_for_security,
+    view_commit,
 )
 
 if get_quart_status():  # pragma: no cover
@@ -101,14 +104,12 @@ if get_quart_status():  # pragma: no cover
 else:
     from flask import make_response, redirect
 
-# Convenient references
-_security = LocalProxy(lambda: current_app.extensions["security"])
-_datastore = LocalProxy(lambda: _security.datastore)
+if t.TYPE_CHECKING:  # pragma: no cover
+    from flask.typing import ResponseValue
 
 
 def default_render_json(payload, code, headers, user):
-    """ Default JSON response handler.
-    """
+    """Default JSON response handler."""
     # Force Content-Type header to json.
     if headers is None:
         headers = dict()
@@ -117,22 +118,12 @@ def default_render_json(payload, code, headers, user):
     return make_response(jsonify(payload), code, headers)
 
 
-PY3 = sys.version_info[0] == 3
-if PY3 and get_quart_status():  # pragma: no cover
-    from .async_compat import _commit  # noqa: F401
-else:
-
-    def _commit(response=None):
-        _datastore.commit()
-        return response
-
-
 def _ctx(endpoint):
     return _security._run_ctx_processor(endpoint)
 
 
 @unauth_csrf(fall_through=True)
-def login():
+def login() -> "ResponseValue":
     """View function for login view
 
     Allow already authenticated users. For GET this is useful for
@@ -155,7 +146,7 @@ def login():
             )
             return _security._render_json(payload, 400, None, None)
         else:
-            return redirect(get_url(_security.post_login_view))
+            return redirect(get_url(cv("POST_LOGIN_VIEW")))
 
     form_class = _security.login_form
 
@@ -170,16 +161,27 @@ def login():
 
     if form.validate_on_submit():
         remember_me = form.remember.data if "remember" in form else None
-        if config_value("TWO_FACTOR") and (
-            config_value("TWO_FACTOR_REQUIRED")
-            or (form.user.tf_totp_secret and form.user.tf_primary_method)
-        ):
-            return tf_login(
-                form.user, remember=remember_me, primary_authn_via="password"
+        if cv("TWO_FACTOR"):
+            if request.is_json and request.content_length:
+                tf_validity_token = request.get_json().get(  # type: ignore
+                    "tf_validity_token", None
+                )
+            else:
+                tf_validity_token = request.cookies.get("tf_validity", default=None)
+
+            tf_validity_token_is_valid = tf_verify_validility_token(
+                tf_validity_token, form.user.fs_uniquifier
             )
 
+            if cv("TWO_FACTOR_REQUIRED") or (is_tf_setup(form.user)):
+                if cv("TWO_FACTOR_ALWAYS_VALIDATE") or (not tf_validity_token_is_valid):
+
+                    return tf_login(
+                        form.user, remember=remember_me, primary_authn_via="password"
+                    )
+
         login_user(form.user, remember=remember_me, authn_via=["password"])
-        after_this_request(_commit)
+        after_this_request(view_commit)
 
         if _security._want_json(request):
             return base_render_json(form, include_auth_token=True)
@@ -191,17 +193,24 @@ def login():
         return base_render_json(form)
 
     if current_user.is_authenticated:
-        return redirect(get_url(_security.post_login_view))
+        return redirect(get_url(cv("POST_LOGIN_VIEW")))
     else:
+        if form.requires_confirmation and cv("REQUIRES_CONFIRMATION_ERROR_VIEW"):
+            do_flash(*get_message("CONFIRMATION_REQUIRED"))
+            return redirect(
+                get_url(
+                    cv("REQUIRES_CONFIRMATION_ERROR_VIEW"),
+                    qparams={"email": form.email.data},
+                )
+            )
         return _security.render_template(
-            config_value("LOGIN_USER_TEMPLATE"), login_user_form=form, **_ctx("login")
+            cv("LOGIN_USER_TEMPLATE"), login_user_form=form, **_ctx("login")
         )
 
 
-@auth_required()
+@auth_required(lambda: cv("API_ENABLED_METHODS"))
 def verify():
-    """View function which handles a authentication verification request.
-    """
+    """View function which handles a authentication verification request."""
     form_class = _security.verify_form
 
     if request.is_json:
@@ -211,7 +220,7 @@ def verify():
 
     if form.validate_on_submit():
         # form may have called verify_and_update_password()
-        after_this_request(_commit)
+        after_this_request(view_commit)
 
         # verified - so set freshness time.
         session["fs_paa"] = time.time()
@@ -226,7 +235,7 @@ def verify():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("VERIFY_TEMPLATE"), verify_form=form, **_ctx("verify")
+        cv("VERIFY_TEMPLATE"), verify_form=form, **_ctx("verify")
     )
 
 
@@ -245,7 +254,7 @@ def logout():
 
 
 @anonymous_user_required
-def register():
+def register() -> "ResponseValue":
     """View function which handles a registration request."""
 
     # For some unknown historic reason - if you don't require confirmation
@@ -258,7 +267,7 @@ def register():
         form_class = _security.register_form
 
     if request.is_json:
-        form_data = MultiDict(request.get_json())
+        form_data: ImmutableMultiDict = ImmutableMultiDict(request.get_json())
     else:
         form_data = request.form
 
@@ -272,10 +281,10 @@ def register():
         # an application that would want random email accounts. It has been like this
         # since the beginning. Note that we still enforce 2FA - however for unified
         # signin - we adhere to historic behavior.
-        if not _security.confirmable or _security.login_without_confirmation:
-            if config_value("TWO_FACTOR") and config_value("TWO_FACTOR_REQUIRED"):
+        if not _security.confirmable or cv("LOGIN_WITHOUT_CONFIRMATION"):
+            if cv("TWO_FACTOR") and cv("TWO_FACTOR_REQUIRED"):
                 return tf_login(user, primary_authn_via="register")
-            after_this_request(_commit)
+            after_this_request(view_commit)
             login_user(user, authn_via=["register"])
             did_login = True
 
@@ -288,9 +297,9 @@ def register():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("REGISTER_USER_TEMPLATE"),
+        cv("REGISTER_USER_TEMPLATE"),
         register_user_form=form,
-        **_ctx("register")
+        **_ctx("register"),
     )
 
 
@@ -314,7 +323,7 @@ def send_login():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("SEND_LOGIN_TEMPLATE"), send_login_form=form, **_ctx("send_login")
+        cv("SEND_LOGIN_TEMPLATE"), send_login_form=form, **_ctx("send_login")
     )
 
 
@@ -349,7 +358,7 @@ def token_login(token):
         return redirect(url_for_security("login"))
 
     login_user(user, authn_via=["token"])
-    after_this_request(_commit)
+    after_this_request(view_commit)
     if _security.redirect_behavior == "spa":
         return redirect(
             get_url(_security.post_login_view, qparams=user.get_redirect_qparams())
@@ -380,9 +389,9 @@ def send_confirmation():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("SEND_CONFIRMATION_TEMPLATE"),
+        cv("SEND_CONFIRMATION_TEMPLATE"),
         send_confirmation_form=form,
-        **_ctx("send_confirmation")
+        **_ctx("send_confirmation"),
     )
 
 
@@ -429,16 +438,16 @@ def confirm_email(token):
         )
 
     confirm_user(user)
-    after_this_request(_commit)
+    after_this_request(view_commit)
 
     if user != current_user:
         logout_user()
-        if config_value("AUTO_LOGIN_AFTER_CONFIRM"):
+        if cv("AUTO_LOGIN_AFTER_CONFIRM"):
             # N.B. this is a (small) security risk if email went to wrong place.
             # and you have the LOGIN_WITH_CONFIRMATION flag since in that case
             # you can be logged in and doing stuff - but another person could
             # get the email.
-            if config_value("TWO_FACTOR") and config_value("TWO_FACTOR_REQUIRED"):
+            if cv("TWO_FACTOR") and cv("TWO_FACTOR_REQUIRED"):
                 return tf_login(user, primary_authn_via="confirm")
             login_user(user, authn_via=["confirm"])
 
@@ -454,7 +463,7 @@ def confirm_email(token):
         get_url(_security.post_confirm_view)
         or get_url(
             _security.post_login_view
-            if config_value("AUTO_LOGIN_AFTER_CONFIRM")
+            if cv("AUTO_LOGIN_AFTER_CONFIRM")
             else _security.login_url
         )
     )
@@ -480,10 +489,19 @@ def forgot_password():
     if _security._want_json(request):
         return base_render_json(form, include_user=False)
 
+    if form.requires_confirmation and _security.requires_confirmation_error_view:
+        do_flash(*get_message("CONFIRMATION_REQUIRED"))
+        return redirect(
+            get_url(
+                _security.requires_confirmation_error_view,
+                qparams={"email": form.email.data},
+            )
+        )
+
     return _security.render_template(
-        config_value("FORGOT_PASSWORD_TEMPLATE"),
+        cv("FORGOT_PASSWORD_TEMPLATE"),
         forgot_password_form=form,
-        **_ctx("forgot_password")
+        **_ctx("forgot_password"),
     )
 
 
@@ -548,10 +566,10 @@ def reset_password(token):
             )
         # for forms - render the reset password form
         return _security.render_template(
-            config_value("RESET_PASSWORD_TEMPLATE"),
+            cv("RESET_PASSWORD_TEMPLATE"),
             reset_password_form=form,
             reset_password_token=token,
-            **_ctx("reset_password")
+            **_ctx("reset_password"),
         )
 
     # This is the POST case.
@@ -579,17 +597,17 @@ def reset_password(token):
             return redirect(url_for_security("forgot_password"))
 
     if form.validate_on_submit():
-        after_this_request(_commit)
+        after_this_request(view_commit)
         update_password(user, form.password.data)
-        if config_value("TWO_FACTOR") and (
-            config_value("TWO_FACTOR_REQUIRED")
+        if cv("TWO_FACTOR") and (
+            cv("TWO_FACTOR_REQUIRED")
             or (form.user.tf_totp_secret and form.user.tf_primary_method)
         ):
             return tf_login(user, primary_authn_via="reset")
         login_user(user, authn_via=["reset"])
         if _security._want_json(request):
-            login_form = _security.login_form(MultiDict({"email": user.email}))
-            setattr(login_form, "user", user)
+            login_form = _security.login_form()
+            login_form.user = user
             return base_render_json(login_form, include_auth_token=True)
         else:
             do_flash(*get_message("PASSWORD_RESET"))
@@ -602,14 +620,14 @@ def reset_password(token):
     if _security._want_json(request):
         return base_render_json(form)
     return _security.render_template(
-        config_value("RESET_PASSWORD_TEMPLATE"),
+        cv("RESET_PASSWORD_TEMPLATE"),
         reset_password_form=form,
         reset_password_token=token,
-        **_ctx("reset_password")
+        **_ctx("reset_password"),
     )
 
 
-@auth_required("basic", "token", "session")
+@auth_required(lambda: cv("API_ENABLED_METHODS"))
 def change_password():
     """View function which handles a change password request."""
 
@@ -621,7 +639,7 @@ def change_password():
         form = form_class(meta=suppress_form_csrf())
 
     if form.validate_on_submit():
-        after_this_request(_commit)
+        after_this_request(view_commit)
         change_user_password(current_user._get_current_object(), form.new_password.data)
         if _security._want_json(request):
             form.user = current_user
@@ -637,9 +655,9 @@ def change_password():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("CHANGE_PASSWORD_TEMPLATE"),
+        cv("CHANGE_PASSWORD_TEMPLATE"),
         change_password_form=form,
-        **_ctx("change_password")
+        **_ctx("change_password"),
     )
 
 
@@ -656,8 +674,7 @@ def two_factor_setup():
     3) user wanting to enable or disable 2FA (assuming application doesn't require it)
 
     In order to CHANGE/ENABLE/DISABLE a 2FA information, user must be properly logged in
-    AND must perform a fresh password validation by
-    calling POST /tf-confirm (which sets 'tf_confirmed' in the session).
+    AND have a 'fresh' authentication.
 
     For initial login when 2FA required of course user can't be logged in - in this
     case we need to have been sent some
@@ -667,13 +684,16 @@ def two_factor_setup():
     form_class = _security.two_factor_setup_form
 
     if request.is_json:
-        form = form_class(MultiDict(request.get_json()), meta=suppress_form_csrf())
+        if request.content_length:
+            form = form_class(MultiDict(request.get_json()), meta=suppress_form_csrf())
+        else:
+            form = form_class(formdata=None, meta=suppress_form_csrf())
     else:
         form = form_class(meta=suppress_form_csrf())
 
     if not current_user.is_authenticated:
         # This is the initial login case
-        # We can also get here from setup if they want to change
+        # We can also get here from setup if they want to change TODO: how?
         if not all(k in session for k in ["tf_user_id", "tf_state"]) or session[
             "tf_state"
         ] not in ["setup_from_login", "validating_profile"]:
@@ -681,18 +701,21 @@ def two_factor_setup():
             tf_clean_session()
             return _tf_illegal_state(form, _security.login_url)
 
-        user = _datastore.get_user(session["tf_user_id"])
+        user = _datastore.find_user(fs_uniquifier=session["tf_user_id"])
         if not user:
             tf_clean_session()
             return _tf_illegal_state(form, _security.login_url)
 
     else:
-        # all other cases require user to be logged in and have performed
-        # additional password verification as signified by 'tf_confirmed'
-        # in the session.
-        if "tf_confirmed" not in session:
-            tf_clean_session()
-            return _tf_illegal_state(form, _security.two_factor_confirm_url)
+        # Caller is changing their TFA profile. This requires a 'fresh' authentication
+        if not check_and_update_authn_fresh(
+            cv("FRESHNESS"),
+            cv("FRESHNESS_GRACE_PERIOD"),
+            get_request_attr("fs_authn_via"),
+        ):
+            return _security._reauthn_handler(
+                cv("FRESHNESS"), cv("FRESHNESS_GRACE_PERIOD")
+            )
         user = current_user
 
     if form.validate_on_submit():
@@ -706,9 +729,9 @@ def two_factor_setup():
         pm = form.setup.data
         if pm == "disable":
             tf_disable(user)
-            after_this_request(_commit)
-            do_flash(*get_message("TWO_FACTOR_DISABLED"))
+            after_this_request(view_commit)
             if not _security._want_json(request):
+                do_flash(*get_message("TWO_FACTOR_DISABLED"))
                 return redirect(get_url(_security.post_login_view))
             else:
                 return base_render_json(form)
@@ -720,11 +743,17 @@ def two_factor_setup():
 
         session["tf_primary_method"] = pm
         session["tf_state"] = "validating_profile"
-        new_phone = form.phone.data if len(form.phone.data) > 0 else None
+        json_response = {
+            "tf_state": "validating_profile",
+            "tf_primary_method": pm,
+        }
+        new_phone = (
+            form.phone.data if (form.phone.data and len(form.phone.data) > 0) else None
+        )
         if new_phone:
             user.tf_phone_number = new_phone
             _datastore.put(user)
-            after_this_request(_commit)
+            after_this_request(view_commit)
 
         # This form is sort of bizarre - for SMS and authenticator
         # you select, then get more info, and submit again.
@@ -744,36 +773,62 @@ def two_factor_setup():
                     return base_render_json(
                         form, include_user=False, error_status_code=500
                     )
+
+        qrcode_values = dict()
+        if pm == "authenticator":
+            authr_setup_values = _security._totp_factory.fetch_setup_values(
+                session["tf_totp_secret"], user
+            )
+            # Add all the values used in qrcode to json response
+            json_response["tf_authr_key"] = authr_setup_values["key"]
+            json_response["tf_authr_username"] = authr_setup_values["username"]
+            json_response["tf_authr_issuer"] = authr_setup_values["issuer"]
+
+            qrcode_values = dict(
+                authr_qrcode=authr_setup_values["image"],
+                authr_key=authr_setup_values["key"],
+                authr_username=authr_setup_values["username"],
+                authr_issuer=authr_setup_values["issuer"],
+            )
         code_form = _security.two_factor_verify_code_form()
         if not _security._want_json(request):
             return _security.render_template(
-                config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+                cv("TWO_FACTOR_SETUP_TEMPLATE"),
                 two_factor_setup_form=form,
                 two_factor_verify_code_form=code_form,
-                choices=config_value("TWO_FACTOR_ENABLED_METHODS"),
+                choices=cv("TWO_FACTOR_ENABLED_METHODS"),
                 chosen_method=pm,
-                **_ctx("tf_setup")
+                **qrcode_values,
+                **_ctx("tf_setup"),
             )
+        return base_render_json(form, include_user=False, additional=json_response)
 
     # We get here on GET and POST with failed validation.
     # For things like phone number - we've already done one POST
-    # that succeeded and now if failed - so retain the initial info
-    if _security._want_json(request):
-        return base_render_json(form, include_user=False)
-
-    code_form = _security.two_factor_verify_code_form()
-    choices = config_value("TWO_FACTOR_ENABLED_METHODS")
-    if not config_value("TWO_FACTOR_REQUIRED"):
+    # that succeeded and now it failed - so retain the initial info
+    choices = cv("TWO_FACTOR_ENABLED_METHODS")
+    if not cv("TWO_FACTOR_REQUIRED"):
         choices.append("disable")
 
+    if _security._want_json(request):
+        # Provide information application/UI might need to render their own form/input
+        json_response = {
+            "tf_required": cv("TWO_FACTOR_REQUIRED"),
+            "tf_primary_method": getattr(user, "tf_primary_method", None),
+            "tf_phone_number": getattr(user, "tf_phone_number", None),
+            "tf_available_methods": choices,
+        }
+        return base_render_json(form, include_user=False, additional=json_response)
+
+    code_form = _security.two_factor_verify_code_form()
     return _security.render_template(
-        config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+        cv("TWO_FACTOR_SETUP_TEMPLATE"),
         two_factor_setup_form=form,
         two_factor_verify_code_form=code_form,
         choices=choices,
         chosen_method=form.setup.data,
-        two_factor_required=config_value("TWO_FACTOR_REQUIRED"),
-        **_ctx("tf_setup")
+        two_factor_required=cv("TWO_FACTOR_REQUIRED"),
+        **_ctx("tf_setup"),
     )
 
 
@@ -786,7 +841,6 @@ def two_factor_token_validation():
        In this case - user not logged in -
        but 'tf_state' == 'ready' or 'validating_profile'
     2) validating after CHANGE/ENABLE 2FA. In this case user logged in/authenticated
-       they must have 'tf_confirmed' set meaning they re-entered their passwd
 
     """
 
@@ -799,20 +853,22 @@ def two_factor_token_validation():
 
     changing = current_user.is_authenticated
     if not changing:
-        # This is the normal login case
+        # This is the normal login case OR initial setup
         if (
             not all(k in session for k in ["tf_user_id", "tf_state"])
             or session["tf_state"] not in ["ready", "validating_profile"]
             or (
                 session["tf_state"] == "validating_profile"
-                and "tf_primary_method" not in session
+                and not all(
+                    k in session for k in ["tf_primary_method", "tf_totp_secret"]
+                )
             )
         ):
             # illegal call on this endpoint
             tf_clean_session()
             return _tf_illegal_state(form, _security.login_url)
 
-        user = _datastore.get_user(session["tf_user_id"])
+        user = _datastore.find_user(fs_uniquifier=session["tf_user_id"])
         form.user = user
         if not user:
             tf_clean_session()
@@ -825,10 +881,9 @@ def two_factor_token_validation():
             pm = session["tf_primary_method"]
             totp_secret = session["tf_totp_secret"]
     else:
+        # Changing TFA profile - user is already authenticated.
         if (
-            not all(
-                k in session for k in ["tf_confirmed", "tf_state", "tf_primary_method"]
-            )
+            not all(k in session for k in ["tf_state", "tf_primary_method"])
             or session["tf_state"] != "validating_profile"
         ):
             tf_clean_session()
@@ -839,18 +894,35 @@ def two_factor_token_validation():
         totp_secret = session["tf_totp_secret"]
         form.user = current_user
 
-    setattr(form, "primary_method", pm)
-    setattr(form, "tf_totp_secret", totp_secret)
+    form.primary_method = pm
+    form.tf_totp_secret = totp_secret
     if form.validate_on_submit():
         # Success - log in user and clear all session variables
+        remember = session.pop("tf_remember_login", None)
         completion_message = complete_two_factor_process(
-            form.user, pm, totp_secret, changing, session.pop("tf_remember_login", None)
+            form.user, pm, totp_secret, changing, remember
         )
-        after_this_request(_commit)
+
+        after_this_request(view_commit)
+
         if not _security._want_json(request):
+            after_this_request(
+                partial(
+                    tf_set_validity_token_cookie,
+                    fs_uniquifier=form.user.fs_uniquifier,
+                    remember=remember,
+                )
+            )
             do_flash(*get_message(completion_message))
+
             return redirect(get_post_login_redirect())
 
+        if (not cv("TWO_FACTOR_ALWAYS_VALIDATE") and remember) and _security._want_json(
+            request
+        ):
+            token = generate_tf_validity_token(form.user.fs_uniquifier)
+            json_response = {"tf_validity_token": token}
+            return base_render_json(form, additional=json_response)
     # GET or not successful POST
     if _security._want_json(request):
         return base_render_json(form)
@@ -860,11 +932,11 @@ def two_factor_token_validation():
         setup_form = _security.two_factor_setup_form()
 
         return _security.render_template(
-            config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+            cv("TWO_FACTOR_SETUP_TEMPLATE"),
             two_factor_setup_form=setup_form,
             two_factor_verify_code_form=form,
-            choices=config_value("TWO_FACTOR_ENABLED_METHODS"),
-            **_ctx("tf_setup")
+            choices=cv("TWO_FACTOR_ENABLED_METHODS"),
+            **_ctx("tf_setup"),
         )
 
     # if we were trying to validate an existing method
@@ -872,18 +944,18 @@ def two_factor_token_validation():
         rescue_form = _security.two_factor_rescue_form()
 
         return _security.render_template(
-            config_value("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
+            cv("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
             two_factor_rescue_form=rescue_form,
             two_factor_verify_code_form=form,
             problem=None,
-            **_ctx("tf_token_validation")
+            **_ctx("tf_token_validation"),
         )
 
 
 @anonymous_user_required
 @unauth_csrf(fall_through=True)
 def two_factor_rescue():
-    """ Function that handles a situation where user can't
+    """Function that handles a situation where user can't
     enter his two-factor validation code
 
     User must have already provided valid username/password.
@@ -905,7 +977,7 @@ def two_factor_rescue():
         tf_clean_session()
         return _tf_illegal_state(form, _security.login_url)
 
-    user = _datastore.get_user(session["tf_user_id"])
+    user = _datastore.find_user(fs_uniquifier=session["tf_user_id"])
     form.user = user
     if not user:
         tf_clean_session()
@@ -932,9 +1004,9 @@ def two_factor_rescue():
                     )
         # send app provider a mail message regarding trouble
         elif problem == "no_mail_access":
-            _security._send_mail(
-                config_value("EMAIL_SUBJECT_TWO_FACTOR_RESCUE"),
-                config_value("TWO_FACTOR_RESCUE_MAIL"),
+            send_mail(
+                cv("EMAIL_SUBJECT_TWO_FACTOR_RESCUE"),
+                cv("TWO_FACTOR_RESCUE_MAIL"),
                 "two_factor_rescue",
                 user=form.user,
             )
@@ -946,99 +1018,12 @@ def two_factor_rescue():
 
     code_form = _security.two_factor_verify_code_form()
     return _security.render_template(
-        config_value("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
+        cv("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
         two_factor_verify_code_form=code_form,
         two_factor_rescue_form=form,
-        rescue_mail=config_value("TWO_FACTOR_RESCUE_MAIL"),
+        rescue_mail=cv("TWO_FACTOR_RESCUE_MAIL"),
         problem=rproblem,
-        **_ctx("tf_token_validation")
-    )
-
-
-@auth_required("basic", "session", "token")
-def two_factor_verify_password():
-    """View function which handles a password verification request."""
-    form_class = _security.two_factor_verify_password_form
-
-    if request.is_json:
-        form = form_class(MultiDict(request.get_json()), meta=suppress_form_csrf())
-    else:
-        form = form_class(meta=suppress_form_csrf())
-
-    if form.validate_on_submit():
-        # form called verify_and_update_password()
-        after_this_request(_commit)
-        session["tf_confirmed"] = True
-        m, c = get_message("TWO_FACTOR_PASSWORD_CONFIRMATION_DONE")
-        if not _security._want_json(request):
-            do_flash(m, c)
-            return redirect(url_for_security("two_factor_setup"))
-        else:
-            return _security._render_json(json_error_response(m), 400, None, None)
-
-    if _security._want_json(request):
-        assert form.user == current_user
-        # form.user = current_user
-        return base_render_json(form)
-
-    return _security.render_template(
-        config_value("TWO_FACTOR_VERIFY_PASSWORD_TEMPLATE"),
-        two_factor_verify_password_form=form,
-        **_ctx("tf_verify_password")
-    )
-
-
-@unauth_csrf(fall_through=True)
-def two_factor_qrcode():
-    if current_user.is_authenticated:
-        user = current_user
-    else:
-        if "tf_user_id" not in session:
-            abort(404)
-        user = _datastore.get_user(session["tf_user_id"])
-        if not user:
-            # Seems like we should be careful here if user_id is gone.
-            tf_clean_session()
-            abort(404)
-
-    if "authenticator" not in config_value("TWO_FACTOR_ENABLED_METHODS"):
-        return abort(404)
-    if (
-        "tf_primary_method" not in session
-        or session["tf_primary_method"] != "authenticator"
-    ):
-        return abort(404)
-
-    totp = user.tf_totp_secret
-    if "tf_totp_secret" in session:
-        totp = session["tf_totp_secret"]
-    try:
-        import pyqrcode
-
-        # By convention, the URI should have the username that the user
-        # logs in with.
-        username = user.calc_username()
-        url = pyqrcode.create(
-            _security._totp_factory.get_totp_uri(
-                username if username else "Unknown", totp
-            )
-        )
-    except ImportError:
-        # For TWO_FACTOR - this should have been checked at app init.
-        raise
-    from io import BytesIO
-
-    stream = BytesIO()
-    url.svg(stream, scale=3)
-    return (
-        stream.getvalue(),
-        200,
-        {
-            "Content-Type": "image/svg+xml",
-            "Cache-Control": "no-cache, no-store, must-revalidate",
-            "Pragma": "no-cache",
-            "Expires": "0",
-        },
+        **_ctx("tf_token_validation"),
     )
 
 
@@ -1075,13 +1060,14 @@ def create_blueprint(app, state, import_name, json_encoder=None):
             state.login_url + slash_url_suffix(state.login_url, "<token>"),
             endpoint="token_login",
         )(token_login)
-    elif config_value("US_SIGNIN_REPLACES_LOGIN", app=app):
+    elif cv("US_SIGNIN_REPLACES_LOGIN", app=app):
         bp.route(state.login_url, methods=["GET", "POST"], endpoint="login")(us_signin)
 
     else:
         bp.route(state.login_url, methods=["GET", "POST"], endpoint="login")(login)
 
-    bp.route(state.verify_url, methods=["GET", "POST"], endpoint="verify")(verify)
+    if cv("FRESHNESS", app=app).total_seconds() >= 0:
+        bp.route(state.verify_url, methods=["GET", "POST"], endpoint="verify")(verify)
 
     if state.unified_signin:
         bp.route(state.us_signin_url, methods=["GET", "POST"], endpoint="us_signin")(
@@ -1102,7 +1088,7 @@ def create_blueprint(app, state, import_name, json_encoder=None):
         )(us_setup_validate)
 
         # Freshness verification
-        if config_value("FRESHNESS", app=app).total_seconds() >= 0:
+        if cv("FRESHNESS", app=app).total_seconds() >= 0:
             bp.route(
                 state.us_verify_url, methods=["GET", "POST"], endpoint="us_verify"
             )(us_verify)
@@ -1115,14 +1101,9 @@ def create_blueprint(app, state, import_name, json_encoder=None):
         bp.route(state.us_verify_link_url, methods=["GET"], endpoint="us_verify_link")(
             us_verify_link
         )
-        bp.route(
-            state.us_qrcode_url + slash_url_suffix(state.us_setup_url, "<token>"),
-            endpoint="us_qrcode",
-        )(us_qrcode)
 
     if state.two_factor:
         tf_token_validation = "two_factor_token_validation"
-        tf_qrcode = "two_factor_qrcode"
         bp.route(
             state.two_factor_setup_url,
             methods=["GET", "POST"],
@@ -1133,17 +1114,11 @@ def create_blueprint(app, state, import_name, json_encoder=None):
             methods=["GET", "POST"],
             endpoint=tf_token_validation,
         )(two_factor_token_validation)
-        bp.route(state.two_factor_qrcode_url, endpoint=tf_qrcode)(two_factor_qrcode)
         bp.route(
             state.two_factor_rescue_url,
             methods=["GET", "POST"],
             endpoint="two_factor_rescue",
         )(two_factor_rescue)
-        bp.route(
-            state.two_factor_confirm_url,
-            methods=["GET", "POST"],
-            endpoint="two_factor_verify_password",
-        )(two_factor_verify_password)
 
     if state.registerable:
         bp.route(state.register_url, methods=["GET", "POST"], endpoint="register")(
