@@ -11,6 +11,7 @@
 import os
 import tempfile
 import time
+import typing as t
 from datetime import datetime
 from urllib.parse import urlsplit
 
@@ -54,9 +55,12 @@ except ImportError:
     except ImportError:
         NO_BABEL = True
 
+if t.TYPE_CHECKING:  # pragma: no cover
+    from flask.testing import FlaskClient
+
 
 @pytest.fixture()
-def app(request):
+def app(request: pytest.FixtureRequest) -> "Flask":
     app = Flask(__name__)
     app.response_class = Response
     app.debug = True
@@ -97,16 +101,19 @@ def app(request):
     else:
         marker_getter = request.keywords.get
     settings = marker_getter("settings")
-    babel = marker_getter("babel")
     if settings is not None:
         for key, value in settings.kwargs.items():
             app.config["SECURITY_" + key.upper()] = value
 
-    mail = Mail(app)
-    if not NO_BABEL and (babel is None or babel.args[0]):
-        Babel(app)
+    app.mail = Mail(app)  # type: ignore
     app.json_encoder = JSONEncoder
-    app.mail = mail
+
+    # use babel marker to signify tests that need babel extension.
+    babel = marker_getter("babel")
+    if babel:
+        if NO_BABEL:
+            raise pytest.skip("Requires Babel")
+        Babel(app)
 
     @app.route("/")
     def index():
@@ -220,6 +227,17 @@ def app(request):
         else:
             return render_template("index.html", content="Fresh Only")
 
+    def revert_forms():
+        # Some forms/tests have dynamic fields - be sure to revert them.
+        if hasattr(app, "security"):
+            if hasattr(app.security.login_form, "email"):
+                del app.security.login_form.email
+            if hasattr(app.security.register_form, "username"):
+                del app.security.register_form.username
+            if hasattr(app.security.confirm_register_form, "username"):
+                del app.security.confirm_register_form.username
+
+    request.addfinalizer(revert_forms)
     return app
 
 
@@ -453,7 +471,7 @@ def peewee_setup(request, app, tmpdir, realdburl):
 
     if realdburl:
         engine_mapper = {
-            "postgres": "peewee.PostgresqlDatabase",
+            "postgresql": "peewee.PostgresqlDatabase",
             "mysql": "peewee.MySQLDatabase",
         }
         db_url, db_info = _setup_realdb(realdburl)
@@ -548,7 +566,7 @@ def pony_setup(request, app, tmpdir, realdburl):
     class Role(db.Entity):
         name = Required(str, unique=True)
         description = Optional(str, nullable=True)
-        users = Set(lambda: User)
+        users = Set(lambda: User)  # type: ignore
 
     class User(db.Entity):
         email = Required(str)
@@ -576,8 +594,10 @@ def pony_setup(request, app, tmpdir, realdburl):
     if realdburl:
         db_url, db_info = _setup_realdb(realdburl)
         pieces = urlsplit(db_url)
+        provider = pieces.scheme.split("+")[0]
+        provider = "postgres" if provider == "postgresql" else provider
         db.bind(
-            provider=pieces.scheme.split("+")[0],
+            provider=provider,
             user=pieces.username,
             password=pieces.password,
             host=pieces.hostname,
@@ -600,9 +620,12 @@ def pony_setup(request, app, tmpdir, realdburl):
 
 
 @pytest.fixture()
-def sqlalchemy_app(app, sqlalchemy_datastore):
-    def create():
-        app.security = Security(app, datastore=sqlalchemy_datastore)
+def sqlalchemy_app(
+    app: "Flask", sqlalchemy_datastore: SQLAlchemyUserDatastore
+) -> t.Callable[[], "Flask"]:
+    def create() -> "Flask":
+        security = Security(app, datastore=sqlalchemy_datastore)
+        app.security = security  # type: ignore
         return app
 
     return create
@@ -645,7 +668,7 @@ def pony_app(app, pony_datastore):
 
 
 @pytest.fixture()
-def client(request, sqlalchemy_app):
+def client(request: pytest.FixtureRequest, sqlalchemy_app: t.Callable) -> "FlaskClient":
     app = sqlalchemy_app()
     populate_data(app)
     return app.test_client()
@@ -689,7 +712,7 @@ def in_app_context(request, sqlalchemy_app):
 
 
 @pytest.fixture()
-def get_message(app):
+def get_message(app: "Flask") -> t.Callable[..., bytes]:
     def fn(key, **kwargs):
         rv = app.config["SECURITY_MSG_" + key][0] % kwargs
         return rv.encode("utf-8")
@@ -730,7 +753,7 @@ def script_info(app, sqlalchemy_datastore):
     def create_app():
         uia = [
             {"email": {"mapper": uia_email_mapper}},
-            {"username": {"mapper": lambda x: x}},
+            {"us_phone_number": {"mapper": lambda x: x}},
         ]
 
         app.config.update(**{"SECURITY_USER_IDENTITY_ATTRIBUTES": uia})
@@ -746,7 +769,7 @@ def pytest_addoption(parser):
         action="store",
         default=None,
         help="""Set url for using real database for testing.
-        For postgres: 'postgres://user:password@host/')""",
+        For postgres: 'postgresql://user:password@host/')""",
     )
 
 
@@ -758,7 +781,7 @@ def realdburl(request):
     compared to sqlite
     To use postgres you need to of course run a postgres instance on localhost
     then pass in an extra arg to pytest:
-    --realdburl postgres://<user>@localhost/
+    --realdburl postgresql://<user>@localhost/
     For mysql same - just download and add a root password.
     --realdburl "mysql+pymysql://root:<password>@localhost/"
     """
