@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     test_csrf
     ~~~~~~~~~~~~~~~~~
@@ -16,7 +15,7 @@ import flask_wtf.csrf
 import pytest
 from flask_wtf import CSRFProtect
 
-from utils import get_session, logout
+from tests.test_utils import get_session, logout
 
 
 REAL_VALIDATE_CSRF = None
@@ -24,7 +23,7 @@ REAL_VALIDATE_CSRF = None
 
 @contextmanager
 def mp_validate_csrf():
-    """ Make sure we are really calling CSRF validation and getting correct answer """
+    """Make sure we are really calling CSRF validation and getting correct answer"""
     orig_validate_csrf = flask_wtf.csrf.validate_csrf
     try:
         mp = MpValidateCsrf(orig_validate_csrf)
@@ -34,7 +33,7 @@ def mp_validate_csrf():
         flask_wtf.csrf.validate_csrf = orig_validate_csrf
 
 
-class MpValidateCsrf(object):
+class MpValidateCsrf:
     success = 0
     failure = 0
 
@@ -70,7 +69,7 @@ def json_login(
     use_header=False,
     remember=None,
 ):
-    """ Return tuple (auth_token, csrf_token)
+    """Return tuple (auth_token, csrf_token)
     Note that since this is sent as JSON rather than form that csrfProtect
     won't find token value (since it looks in request.form).
     """
@@ -84,7 +83,7 @@ def json_login(
         data["csrf_token"] = csrf_token
 
     response = client.post(
-        endpoint or "/login" + "?include_auth_token",
+        endpoint or "/login?include_auth_token",
         content_type="application/json",
         json=data,
         headers=headers,
@@ -205,7 +204,7 @@ def test_login_csrf_unauth_double(app, client, get_message):
 
 @pytest.mark.recoverable()
 def test_reset(app, client):
-    """ Test that form-based CSRF works for /reset """
+    """Test that form-based CSRF works for /reset"""
     app.config["WTF_CSRF_ENABLED"] = True
 
     with mp_validate_csrf() as mp:
@@ -222,7 +221,7 @@ def test_reset(app, client):
 
 @pytest.mark.recoverable()
 def test_cp_reset(app, client):
-    """ Test that header based CSRF works for /reset when
+    """Test that header based CSRF works for /reset when
     using WTF_CSRF_CHECK_DEFAULT=False.
     """
     app.config["WTF_CSRF_ENABLED"] = True
@@ -400,9 +399,7 @@ def test_different_mechanisms_nc(app, client_nc):
     assert mp.success == 0 and mp.failure == 0
 
 
-@pytest.mark.settings(
-    csrf_ignore_unauth_endpoints=True, CSRF_COOKIE={"key": "X-XSRF-Token"}
-)
+@pytest.mark.settings(csrf_ignore_unauth_endpoints=True, CSRF_COOKIE_NAME="XSRF-Token")
 def test_csrf_cookie(app, client):
     app.config["WTF_CSRF_ENABLED"] = True
     app.config["WTF_CSRF_CHECK_DEFAULT"] = False
@@ -411,21 +408,23 @@ def test_csrf_cookie(app, client):
     json_login(client)
     found = False
     for cookie in client.cookie_jar:
-        if cookie.name == "X-XSRF-Token":
+        if cookie.name == "XSRF-Token":
             found = True
             assert cookie.path == "/"
+            # Alas http.cookiejar doesn't support samesite...
+            # assert cookie.samesite == "Strict"
     assert found
 
     # Make sure cleared on logout
     response = client.post("/logout", content_type="application/json")
     assert response.status_code == 200
-    assert "X-XSRF-Token" not in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" not in [c.name for c in client.cookie_jar]
 
 
-@pytest.mark.settings(CSRF_COOKIE={"key": "X-XSRF-Token"})
+@pytest.mark.settings(CSRF_COOKIE={"key": "XSRF-Token"})
 @pytest.mark.changeable()
 def test_cp_with_token_cookie(app, client):
-    # Make sure can use returned CSRF-Token cookie in Header.
+    # Make sure can use returned CSRF-Token cookie in Header when changing password.
     app.config["WTF_CSRF_ENABLED"] = True
     CSRFProtect(app)
 
@@ -437,7 +436,7 @@ def test_cp_with_token_cookie(app, client):
         new_password="battery staple",
         new_password_confirm="battery staple",
     )
-    csrf_token = [c.value for c in client.cookie_jar if c.name == "X-XSRF-Token"][0]
+    csrf_token = [c.value for c in client.cookie_jar if c.name == "XSRF-Token"][0]
     with mp_validate_csrf() as mp:
         response = client.post(
             "/change",
@@ -446,15 +445,12 @@ def test_cp_with_token_cookie(app, client):
             headers={"X-XSRF-Token": csrf_token},
         )
         assert response.status_code == 200
-    # 2 successes since the utils:csrf_cookie_handler will check
-    assert mp.success == 2 and mp.failure == 0
+    assert mp.success == 1 and mp.failure == 0
     json_logout(client)
-    assert "X-XSRF-Token" not in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" not in [c.name for c in client.cookie_jar]
 
 
-@pytest.mark.settings(
-    CSRF_COOKIE={"key": "X-XSRF-Token"}, csrf_ignore_unauth_endpoints=True
-)
+@pytest.mark.settings(CSRF_COOKIE_NAME="XSRF-Token", csrf_ignore_unauth_endpoints=True)
 @pytest.mark.changeable()
 def test_cp_with_token_cookie_expire(app, client):
     # Make sure that we get a new Csrf-Token cookie if expired.
@@ -472,7 +468,7 @@ def test_cp_with_token_cookie_expire(app, client):
         new_password="battery staple",
         new_password_confirm="battery staple",
     )
-    csrf_token = [c.value for c in client.cookie_jar if c.name == "X-XSRF-Token"][0]
+    csrf_token = [c.value for c in client.cookie_jar if c.name == "XSRF-Token"][0]
     with mp_validate_csrf() as mp:
         response = client.post(
             "/change",
@@ -484,18 +480,18 @@ def test_cp_with_token_cookie_expire(app, client):
         assert b"expired" in response.data
 
         # Should have gotten a new CSRF cookie value
-        new_csrf_token = [
-            c.value for c in client.cookie_jar if c.name == "X-XSRF-Token"
-        ][0]
+        new_csrf_token = [c.value for c in client.cookie_jar if c.name == "XSRF-Token"][
+            0
+        ]
         assert csrf_token != new_csrf_token
     # 2 failures since the utils:csrf_cookie_handler will check
     assert mp.success == 0 and mp.failure == 2
     json_logout(client)
-    assert "X-XSRF-Token" not in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" not in [c.name for c in client.cookie_jar]
 
 
 @pytest.mark.settings(
-    CSRF_COOKIE={"key": "X-XSRF-Token"}, CSRF_COOKIE_REFRESH_EACH_REQUEST=True
+    CSRF_COOKIE_NAME="XSRF-Token", CSRF_COOKIE_REFRESH_EACH_REQUEST=True
 )
 @pytest.mark.changeable()
 def test_cp_with_token_cookie_refresh(app, client):
@@ -512,7 +508,7 @@ def test_cp_with_token_cookie_refresh(app, client):
         new_password_confirm="battery staple",
     )
 
-    csrf_cookie = [c for c in client.cookie_jar if c.name == "X-XSRF-Token"][0]
+    csrf_cookie = [c for c in client.cookie_jar if c.name == "XSRF-Token"][0]
     with mp_validate_csrf() as mp:
         # Delete cookie - we should always get a new one
         client.delete_cookie(csrf_cookie.domain, csrf_cookie.name)
@@ -523,14 +519,23 @@ def test_cp_with_token_cookie_refresh(app, client):
             headers={"X-XSRF-Token": csrf_cookie.value},
         )
         assert response.status_code == 200
-        csrf_cookie = [c for c in client.cookie_jar if c.name == "X-XSRF-Token"][0]
+        csrf_cookie = [c for c in client.cookie_jar if c.name == "XSRF-Token"][0]
         assert csrf_cookie
     assert mp.success == 1 and mp.failure == 0
+
+    # delete cookie again, do a 'GET' - the REFRESH_COOKIE_ON_EACH_REQUEST should
+    # send us a new one
+    client.delete_cookie(csrf_cookie.domain, csrf_cookie.name)
+    response = client.get("/change")
+    assert response.status_code == 200
+    csrf_cookie = [c for c in client.cookie_jar if c.name == "XSRF-Token"][0]
+    assert csrf_cookie
+
     json_logout(client)
-    assert "X-XSRF-Token" not in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" not in [c.name for c in client.cookie_jar]
 
 
-@pytest.mark.settings(CSRF_COOKIE={"key": "X-XSRF-Token"})
+@pytest.mark.settings(CSRF_COOKIE_NAME="XSRF-Token")
 @pytest.mark.changeable()
 def test_remember_login_csrf_cookie(app, client):
     # Test csrf cookie upon resuming a remember session
@@ -540,7 +545,7 @@ def test_remember_login_csrf_cookie(app, client):
     # Login with remember_token generation
     json_login(client, use_header=True, remember=True)
 
-    csrf_cookie = [c for c in client.cookie_jar if c.name == "X-XSRF-Token"][0]
+    csrf_cookie = [c for c in client.cookie_jar if c.name == "XSRF-Token"][0]
     session_cookie = [c for c in client.cookie_jar if c.name == "session"][0]
     # Delete session and csrf cookie - we should always get new ones
     client.delete_cookie(csrf_cookie.domain, csrf_cookie.name)
@@ -551,9 +556,9 @@ def test_remember_login_csrf_cookie(app, client):
     response = client.get("/profile")
     assert response.status_code == 200
     assert "session" in [c.name for c in client.cookie_jar]
-    assert "X-XSRF-Token" in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" in [c.name for c in client.cookie_jar]
     # Logout and check that everything cleans up nicely
     json_logout(client)
     assert "remember_token" not in [c.name for c in client.cookie_jar]
     assert "session" not in [c.name for c in client.cookie_jar]
-    assert "X-XSRF-Token" not in [c.name for c in client.cookie_jar]
+    assert "XSRF-Token" not in [c.name for c in client.cookie_jar]
