@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     test_datastore
     ~~~~~~~~~~~~~~
@@ -12,9 +11,9 @@
 
 import datetime
 from pytest import raises, skip
-from utils import init_app_with_options, get_num_queries, is_sqlalchemy
+from tests.test_utils import init_app_with_options, get_num_queries, is_sqlalchemy
 
-from flask_security import RoleMixin, Security, UserMixin
+from flask_security import RoleMixin, Security, UserMixin, LoginForm, RegisterForm
 from flask_security.datastore import Datastore, UserDatastore
 
 
@@ -50,8 +49,6 @@ def test_unimplemented_user_datastore_methods():
         datastore.find_user(None)
     with raises(NotImplementedError):
         datastore.find_role(None)
-    with raises(NotImplementedError):
-        datastore.get_user(None)
 
 
 def test_toggle_active():
@@ -94,60 +91,31 @@ def test_activate_returns_false_if_already_true():
     assert not datastore.activate_user(user)
 
 
-def test_get_user(app, datastore):
-    # The order of identity attributes is important for testing.
-    # drivers like psycopg2 will abort the transaction if they throw an
-    # error and not continue on so we want to check that case of passing in
-    # a string for a numeric field and being able to move onto the next
-    # column.
-    init_app_with_options(
-        app,
-        datastore,
-        **{
-            "SECURITY_USER_IDENTITY_ATTRIBUTES": (
-                "email",
-                "security_number",
-                "username",
-            )
-        }
-    )
-
-    with app.app_context():
-        user_id = datastore.find_user(email="matt@lp.com").id
-
-        user = datastore.get_user(user_id)
-        assert user is not None
-
-        user = datastore.get_user("matt@lp.com")
-        assert user is not None
-
-        user = datastore.get_user("matt")
-        assert user is not None
-
-        # Regression check (make sure we don't match wildcards)
-        user = datastore.get_user("%lp.com")
-        assert user is None
-
-        # Verify that numeric non PK works
-        user = datastore.get_user(123456)
-        assert user is not None
-
-
 def test_find_user(app, datastore):
     init_app_with_options(app, datastore)
 
     with app.app_context():
-        user_id = datastore.find_user(email="gene@lp.com").id
+        user_id = datastore.find_user(email="gene@lp.com").fs_uniquifier
 
         current_nqueries = get_num_queries(datastore)
-        assert user_id == datastore.find_user(security_number=889900).id
+        assert user_id == datastore.find_user(security_number=889900).fs_uniquifier
         end_nqueries = get_num_queries(datastore)
         if current_nqueries is not None:
             if is_sqlalchemy(datastore):
                 # This should have done just 1 query across all attrs.
                 assert end_nqueries == (current_nqueries + 1)
 
-        assert user_id == datastore.find_user(username="gene").id
+        assert user_id == datastore.find_user(username="gene").fs_uniquifier
+
+
+def test_find_user_multikey(app, datastore):
+    init_app_with_options(app, datastore)
+
+    with app.app_context():
+        with raises(ValueError):
+            datastore.find_user(
+                case_insensitive=True, email="gene@lp.com", security_number=889900
+            )
 
 
 def test_find_role(app, datastore):
@@ -172,11 +140,6 @@ def test_add_role_to_user(app, datastore):
         assert datastore.add_role_to_user(user, "editor") is False
         assert user.has_role("editor") is True
 
-        # Test with email
-        assert datastore.add_role_to_user("jill@lp.com", "editor") is True
-        user = datastore.find_user(email="jill@lp.com")
-        assert user.has_role("editor") is True
-
         # Test remove role
         assert datastore.remove_role_from_user(user, "editor") is True
         assert datastore.remove_role_from_user(user, "editor") is False
@@ -229,15 +192,24 @@ def test_access_datastore_from_app_factory_pattern(app, datastore):
 
 
 def test_init_app_kwargs_override_constructor_kwargs(app, datastore):
+    class ConLoginForm(LoginForm):
+        pass
+
+    class ConRegisterForm(RegisterForm):
+        pass
+
+    class InitLoginForm(LoginForm):
+        pass
+
     security = Security(
         datastore=datastore,
-        login_form="__init__login_form",
-        register_form="__init__register_form",
+        login_form=ConLoginForm,
+        register_form=ConRegisterForm,
     )
-    security.init_app(app, login_form="init_app_login_form")
+    security.init_app(app, login_form=InitLoginForm)
 
-    assert security.login_form == "init_app_login_form"
-    assert security.register_form == "__init__register_form"
+    assert security.login_form == InitLoginForm
+    assert security.register_form == ConRegisterForm
 
 
 def test_create_user_with_roles_and_permissions(app, datastore):
@@ -325,7 +297,7 @@ def test_modify_permissions(app, datastore):
 
 
 def test_get_permissions(app, datastore):
-    """ Verify that role.permissions = None works. """
+    """Verify that role.permissions = None works."""
     ds = datastore
     if not hasattr(ds.role_model, "permissions"):
         return
@@ -381,7 +353,7 @@ def test_modify_permissions_multi(app, datastore):
 
 
 def test_modify_permissions_unsupported(app, datastore):
-    from conftest import PonyUserDatastore
+    from tests.conftest import PonyUserDatastore
 
     ds = datastore
     if hasattr(datastore.role_model, "permissions"):
@@ -405,7 +377,7 @@ def test_modify_permissions_unsupported(app, datastore):
 
 
 def test_uuid(app, request, tmpdir, realdburl):
-    """ Test that UUID extension of postgresql works as a primary id for users """
+    """Test that UUID extension of postgresql works as a primary id for users"""
     import uuid
     from flask_sqlalchemy import SQLAlchemy
     from sqlalchemy import Boolean, Column, DateTime, Integer, ForeignKey, String
@@ -413,10 +385,10 @@ def test_uuid(app, request, tmpdir, realdburl):
     from sqlalchemy.orm import relationship, backref
 
     from flask_security import SQLAlchemyUserDatastore
-    from conftest import _setup_realdb, _teardown_realdb
+    from tests.conftest import _setup_realdb, _teardown_realdb
 
     # UUID type only supported by postgres - not sqlite.
-    if not realdburl or "postgres" not in realdburl:
+    if not realdburl or "postgresql" not in realdburl:
         skip("This test only works on postgres")
     db_url, db_info = _setup_realdb(realdburl)
     app.config["SQLALCHEMY_DATABASE_URI"] = db_url
@@ -435,9 +407,10 @@ def test_uuid(app, request, tmpdir, realdburl):
             UUID(as_uuid=True), primary_key=True, default=uuid.uuid4, index=True
         )
         email = Column(String(255), unique=True)
+        fs_uniquifier = Column(String(64), unique=True, nullable=False)
         first_name = Column(String(255), index=True)
         last_name = Column(String(255), index=True)
-        username = Column(String(255), unique=True)
+        username = Column(String(255), unique=True, nullable=True)
         password = Column(String(255))
         active = Column(Boolean())
         created_at = Column(DateTime, default=datetime.datetime.utcnow)
@@ -472,41 +445,5 @@ def test_uuid(app, request, tmpdir, realdburl):
     app.security = Security(app, datastore=ds)
 
     with app.app_context():
-        user = ds.get_user("matt@lp.com")
+        user = ds.find_user(email="matt@lp.com")
         assert not user
-
-
-def test_user_loader(app, sqlalchemy_datastore):
-    # user_loader now tries first to match fs_uniquifier then match user.id
-    # While we know that fs_uniquifier is a string - we don't know what user.id is
-    # and we know that psycopg2 is really finicky about this.
-    from flask_security.core import _user_loader
-
-    init_app_with_options(app, sqlalchemy_datastore)
-    with app.app_context():
-        jill = sqlalchemy_datastore.find_user(email="jill@lp.com")
-
-        # normal case
-        user = _user_loader(jill.fs_uniquifier)
-        assert user.email == "jill@lp.com"
-
-        # send in an int - make sure it is cast to string for check against
-        # fs_uniquifier
-        user = _user_loader(1000)
-        assert not user
-        # with psycopg2 this will lock up DB but since we pass exceptions we can
-        # check this by trying some other DB operation
-        jill = sqlalchemy_datastore.find_user(email="jill@lp.com")
-        assert jill
-
-        # This works since DBs seem to try to cast to underlying type
-        user = _user_loader("10")
-        jill = sqlalchemy_datastore.find_user(email="jill@lp.com")
-        assert jill
-
-        # Since this doesn't match an existing fs_uniqifier, and user.id is an int
-        # this will make pyscopg2 angry. This test verifies that we don't in fact
-        # try to take a string and use it to lookup an int.
-        user = _user_loader("someunknown")
-        jill = sqlalchemy_datastore.find_user(email="jill@lp.com")
-        assert jill
