@@ -5,7 +5,7 @@
     Lots of tests
 
     :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2021 by J. Christopher Wagner (jwag).
+    :copyright: (c) 2019-2022 by J. Christopher Wagner (jwag).
     :license: MIT, see LICENSE for more details.
 """
 
@@ -814,8 +814,7 @@ def test_authn_freshness(
         response = client.get("/myspecialview", follow_redirects=False)
         assert response.status_code == 302
         assert (
-            response.location
-            == "http://localhost/verify?next=http%3A%2F%2Flocalhost%2Fmyspecialview"
+            "/verify?next=http%3A%2F%2Flocalhost%2Fmyspecialview" in response.location
         )
     assert flashes[0]["category"] == "error"
     assert flashes[0]["message"].encode("utf-8") == get_message(
@@ -881,6 +880,24 @@ def test_authn_freshness_callable(app, client, get_message):
     assert response.status_code == 200
 
 
+@pytest.mark.settings(url_prefix="/myprefix")
+def test_default_authn_bp(app, client):
+    """Test default reauthn handler with blueprint prefix"""
+
+    @auth_required(within=0.001, grace=0)
+    def myview():
+        return Response(status=200)
+
+    app.add_url_rule("/myview", view_func=myview, methods=["GET"])
+    authenticate(client, endpoint="/myprefix/login")
+
+    # This should require additional authn and redirect to verify
+    time.sleep(0.1)
+    response = client.get("/myview", follow_redirects=False)
+    assert response.status_code == 302
+    assert "/myprefix/verify?next=http%3A%2F%2Flocalhost%2Fmyview" in response.location
+
+
 def test_authn_freshness_grace(app, client, get_message):
     # Test that grace override within.
     @auth_required(within=lambda: timedelta(minutes=30), grace=10)
@@ -920,10 +937,7 @@ def test_authn_freshness_nc(app, client_nc, get_message):
     # This should fail - should be a redirect
     response = client_nc.get("/myview", headers=h, follow_redirects=False)
     assert response.status_code == 302
-    assert (
-        response.location
-        == "http://localhost/verify?next=http%3A%2F%2Flocalhost%2Fmyview"
-    )
+    assert "/verify?next=http%3A%2F%2Flocalhost%2Fmyview" in response.location
 
 
 def test_verify_fresh(app, client, get_message):
@@ -1085,11 +1099,11 @@ def test_post_security_with_application_root(app, sqlalchemy_datastore):
         "/login", data=dict(email="matt@lp.com", password="password")
     )
     assert response.status_code == 302
-    assert response.headers["Location"] == "http://localhost/root"
+    assert "/root" in response.location
 
     response = client.get("/logout")
     assert response.status_code == 302
-    assert response.headers["Location"] == "http://localhost/root"
+    assert "/root" in response.location
 
 
 def test_post_security_with_application_root_and_views(app, sqlalchemy_datastore):
@@ -1108,11 +1122,11 @@ def test_post_security_with_application_root_and_views(app, sqlalchemy_datastore
         "/login", data=dict(email="matt@lp.com", password="password")
     )
     assert response.status_code == 302
-    assert response.headers["Location"] == "http://localhost/post_login"
+    assert "/post_login" in response.location
 
     response = client.get("/logout")
     assert response.status_code == 302
-    assert response.headers["Location"] == "http://localhost/post_logout"
+    assert "/post_logout" in response.location
 
 
 @pytest.mark.settings(redirect_validate_mode="regex")
