@@ -8,8 +8,8 @@
 import pytest
 import re
 from flask import Flask
-import jinja2
-from tests.test_utils import authenticate, check_xlation, logout
+import markupsafe
+from tests.test_utils import authenticate, check_xlation, json_authenticate, logout
 
 from flask_security import Security
 from flask_security.core import UserMixin
@@ -36,14 +36,14 @@ def test_registerable_flag(clients, app, get_message):
 
     # Test registering is successful, sends email, and fires signal
     @user_registered.connect_via(app)
-    def on_user_registered(app, user, confirm_token, form_data):
+    def on_user_registered(app, **kwargs):
 
         assert isinstance(app, Flask)
-        assert isinstance(user, UserMixin)
-        assert confirm_token is None
-        assert len(form_data.keys()) > 0
+        assert isinstance(kwargs["user"], UserMixin)
+        assert kwargs["confirm_token"] is None
+        assert len(kwargs["form_data"].keys()) > 0
 
-        recorded.append(user)
+        recorded.append(kwargs["user"])
 
     data = dict(
         email="dude@lp.com",
@@ -116,19 +116,18 @@ def test_registerable_flag(clients, app, get_message):
 
 
 @pytest.mark.confirmable()
+@pytest.mark.babel()
 def test_xlation(app, client, get_message_local):
     # Test form and email translation
     app.config["BABEL_DEFAULT_LOCALE"] = "fr_FR"
     assert check_xlation(app, "fr_FR"), "You must run python setup.py compile_catalog"
 
     response = client.get("/register", follow_redirects=True)
-    with app.app_context():
+    with app.test_request_context():
         # Check header
-        assert (
-            f'<h1>{localize_callback("Register")}</h1>'.encode("utf-8") in response.data
-        )
+        assert f'<h1>{localize_callback("Register")}</h1>'.encode() in response.data
         submit = localize_callback(_default_field_labels["register"])
-        assert f'value="{submit}"'.encode("utf-8") in response.data
+        assert f'value="{submit}"'.encode() in response.data
 
     with app.mail.record_messages() as outbox:
         response = client.post(
@@ -141,7 +140,7 @@ def test_xlation(app, client, get_message_local):
             follow_redirects=True,
         )
 
-    with app.app_context():
+    with app.test_request_context():
         assert (
             get_message_local("CONFIRM_REGISTRATION", email="me@fr.com").encode("utf-8")
             in response.data
@@ -154,7 +153,7 @@ def test_xlation(app, client, get_message_local):
         )
         assert (
             str(
-                jinja2.escape(
+                markupsafe.escape(
                     localize_callback(
                         "You can confirm your email through the link below:"
                     )
@@ -162,10 +161,6 @@ def test_xlation(app, client, get_message_local):
             )
             in outbox[0].html
         )
-        assert (
-            localize_callback("You can confirm your email through the link below:")
-            in outbox[0].body
-        )
 
 
 @pytest.mark.confirmable()
@@ -306,14 +301,15 @@ def test_form_data_is_passed_to_user_registered_signal(app, sqlalchemy_datastore
     recorded = []
 
     @user_registered.connect_via(app)
-    def on_user_registered(app, user, confirm_token, form_data):
+    def on_user_registered(app, **kwargs):
 
         assert isinstance(app, Flask)
-        assert isinstance(user, UserMixin)
-        assert confirm_token is None
-        assert form_data["additional_field"] == "additional_data"
+        assert isinstance(kwargs["user"], UserMixin)
+        assert kwargs["confirm_token"] is None
+        assert kwargs["confirmation_token"] is None
+        assert kwargs["form_data"]["additional_field"] == "additional_data"
 
-        recorded.append(user)
+        recorded.append(kwargs["user"])
 
     client = app.test_client()
 
@@ -374,34 +370,21 @@ def test_easy_password(app, sqlalchemy_datastore):
     )
 
 
-def test_nullable_username(app, sqlalchemy_datastore):
+@pytest.mark.settings(username_enable=True)
+def test_nullable_username(app, client):
     # sqlalchemy datastore uses fsqlav2 which has username as unique and nullable
     # make sure can register multiple users with no username
     # Note that current WTForms (2.2.1) has a bug where StringFields can never be
     # None - it changes them to an empty string. DBs don't like that if you have
     # your column be 'nullable'.
-    class NullableStringField(StringField):
-        def process_formdata(self, valuelist):
-            if valuelist:
-                self.data = valuelist[0]
-
-    class MyRegisterForm(ConfirmRegisterForm):
-        username = NullableStringField("Username")
-
-    app.config["SECURITY_CONFIRM_REGISTER_FORM"] = MyRegisterForm
-    security = Security(datastore=sqlalchemy_datastore)
-    security.init_app(app)
-
-    client = app.test_client()
-
-    data = dict(email="u1@test.com", password="password", password_confirm="password")
+    data = dict(email="u1@test.com", password="password")
     response = client.post(
         "/register", json=data, headers={"Content-Type": "application/json"}
     )
     assert response.status_code == 200
     logout(client)
 
-    data = dict(email="u2@test.com", password="password", password_confirm="password")
+    data = dict(email="u2@test.com", password="password")
     response = client.post(
         "/register", json=data, headers={"Content-Type": "application/json"}
     )
@@ -459,6 +442,7 @@ def test_email_normalization_options(app, client, get_message):
     assert get_message("INVALID_EMAIL_ADDRESS") in response.data
 
 
+@pytest.mark.babel()
 def test_form_error(app, client, get_message):
     # A few form validations use ValidatorMixin which provides a lazy string
     # Since CLI doesn't render_template it was seeing those lazy strings.
@@ -482,3 +466,138 @@ def test_form_error(app, client, get_message):
             register_user_form=rform,
         )
         assert get_message("EMAIL_NOT_PROVIDED") in rendered.encode("utf-8")
+
+
+@pytest.mark.settings(username_enable=True)
+@pytest.mark.unified_signin()
+def test_username(app, client, get_message):
+    data = dict(
+        email="dude@lp.com",
+        username="dude",
+        password="awesome sunset",
+    )
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.headers["Content-Type"] == "application/json"
+    assert response.status_code == 200
+    logout(client)
+
+    # login using historic - email field...
+    response = json_authenticate(client, email="dude", password="awesome sunset")
+    assert response.status_code == 200
+
+    logout(client)
+
+    # login using us-signin
+    response = client.post(
+        "/us-signin",
+        data=dict(identity="dude", passcode="awesome sunset"),
+        follow_redirects=True,
+    )
+    assert b"Welcome dude@lp.com" in response.data
+
+    # make sure username is unique
+    logout(client)
+    data = dict(
+        email="dude@lp.com",
+        username="dude",
+        password="awesome sunset",
+    )
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        get_message("USERNAME_ALREADY_ASSOCIATED", username="dude")
+        == response.json["response"]["errors"]["username"][0].encode()
+    )
+
+
+@pytest.mark.settings(username_enable=True)
+@pytest.mark.unified_signin()
+def test_username_normalize(app, client, get_message):
+    data = dict(
+        email="dude@lp.com",
+        username="Imnumber\N{ROMAN NUMERAL ONE}",
+        password="awesome sunset",
+    )
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 200
+    logout(client)
+
+    response = client.post(
+        "/us-signin",
+        data=dict(
+            identity="Imnumber\N{LATIN CAPITAL LETTER I}", passcode="awesome sunset"
+        ),
+        follow_redirects=True,
+    )
+    assert b"Welcome dude@lp.com" in response.data
+
+
+@pytest.mark.settings(username_enable=True, username_required=True)
+def test_username_errors(app, client, get_message):
+    data = dict(
+        email="dude@lp.com",
+        username="dud",
+        password="awesome sunset",
+    )
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        "Username must be at least"
+        in response.json["response"]["errors"]["username"][0]
+    )
+
+    data["username"] = "howlongcanIbebeforeyoucomplainIsupposereallyreallylong"
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        "Username must be at least"
+        in response.json["response"]["errors"]["username"][0]
+    )
+
+    # try evil characters
+    data["username"] = "hi <script>doing evil</script>"
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        get_message("USERNAME_ILLEGAL_CHARACTERS")
+        == response.json["response"]["errors"]["username"][0].encode()
+    )
+
+    # No spaces or punctuation
+    data["username"] = "hi there?"
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        get_message("USERNAME_DISALLOWED_CHARACTERS")
+        in response.json["response"]["errors"]["username"][0].encode()
+    )
+
+    data["username"] = None
+    response = client.post(
+        "/register", json=data, headers={"Content-Type": "application/json"}
+    )
+    assert response.status_code == 400
+    assert (
+        get_message("USERNAME_NOT_PROVIDED")
+        == response.json["response"]["errors"]["username"][0].encode()
+    )
+
+
+def test_username_not_enabled(app, client, get_message):
+    response = client.get("/register")
+    assert b"username" not in response.data
+    assert not hasattr(RegisterForm, "username")
