@@ -1,4 +1,4 @@
-# :copyright: (c) 2019-2020 by J. Christopher Wagner (jwag).
+# :copyright: (c) 2019-2021 by J. Christopher Wagner (jwag).
 # :license: MIT, see LICENSE for more details.
 
 """
@@ -22,10 +22,11 @@ data and a mail sender that flashes what mail would be sent!
 """
 
 import datetime
+import typing as t
 import os
 
 from flask import Flask, flash, render_template_string, request, session
-import flask_babelex
+
 from flask.json import JSONEncoder
 from flask_security import (
     Security,
@@ -40,7 +41,7 @@ from flask_security.signals import (
     reset_password_instructions_sent,
     user_registered,
 )
-from flask_security.utils import hash_password
+from flask_security.utils import hash_password, uia_email_mapper, uia_phone_mapper
 
 
 def _find_bool(v):
@@ -51,7 +52,7 @@ def _find_bool(v):
     return v
 
 
-class FlashMail(object):
+class FlashMail:
     def __init__(self, app):
         app.extensions["mail"] = self
 
@@ -60,7 +61,8 @@ class FlashMail(object):
 
 
 def create_app():
-    app = Flask(__name__)
+    # Use real templates - not test templates...
+    app = Flask("view_scaffold", template_folder="../")
     app.config["DEBUG"] = True
     # SECRET_KEY generated using: secrets.token_urlsafe()
     app.config["SECRET_KEY"] = "pf9Wkove4IKEAXvy-cQkeDPhv9Cb3Ag-wyJILbq_dFw"
@@ -69,7 +71,10 @@ def create_app():
 
     app.config["LOGIN_DISABLED"] = False
     app.config["WTF_CSRF_ENABLED"] = False
-    app.config["SECURITY_USER_IDENTITY_ATTRIBUTES"] = ["email", "us_phone_number"]
+    app.config["SECURITY_USER_IDENTITY_ATTRIBUTES"] = [
+        {"email": {"mapper": uia_email_mapper, "case_insensitive": True}},
+        {"us_phone_number": {"mapper": uia_phone_mapper}},
+    ]
     # app.config["SECURITY_US_ENABLED_METHODS"] = ["password"]
     # app.config["SECURITY_US_ENABLED_METHODS"] = ["authenticator", "password"]
 
@@ -78,8 +83,9 @@ def create_app():
     app.config["SECURITY_TOTP_SECRETS"] = {
         "1": "TjQ9Qa31VOrfEzuPy4VHQWPCTmRzCnFzMKLxXYiZu9B"
     }
-    app.config["SECURITY_FRESHNESS"] = datetime.timedelta(minutes=5)
+    app.config["SECURITY_FRESHNESS"] = datetime.timedelta(minutes=0.5)
     app.config["SECURITY_FRESHNESS_GRACE_PERIOD"] = datetime.timedelta(minutes=2)
+    app.config["SECURITY_USERNAME_ENABLE"] = True
 
     # Turn on all features (except passwordless since that removes normal login)
     for opt in [
@@ -107,25 +113,39 @@ def create_app():
     app.json_encoder = JSONEncoder
     # Setup Flask-Security
     user_datastore = SQLAlchemySessionUserDatastore(db_session, User, Role)
-    security = Security(app, user_datastore)
+    Security(app, user_datastore)
 
-    # This is NOT ideal since it basically changes entire APP (which is fine for
-    # this test - but not fine for general use).
-    babel = flask_babelex.Babel(app, default_domain=security.i18n_domain)
+    try:
+        import flask_babel
 
-    @babel.localeselector
-    def get_locale():
-        # For a given session - set lang based on first request.
-        # Honor explicit url request first
-        if "lang" not in session:
-            locale = request.args.get("lang", None)
-            if not locale:
-                locale = request.accept_languages.best
-            if not locale:
-                locale = "en"
-            if locale:
-                session["lang"] = locale
-        return session.get("lang", None).replace("-", "_")
+        babel = flask_babel.Babel(app)
+    except ImportError:
+        try:
+            import flask_babelex
+
+            babel = flask_babelex.Babel(app)
+        except ImportError:
+            babel = None
+
+    if babel:
+
+        @babel.localeselector
+        def get_locale():
+            # For a given session - set lang based on first request.
+            # Honor explicit url request first
+            if "lang" not in session:
+                locale = request.args.get("lang", None)
+                if not locale:
+                    locale = request.accept_languages.best
+                if not locale:
+                    locale = "en"
+                if locale:
+                    session["lang"] = locale
+            return session.get("lang", None).replace("-", "_")
+
+    @app.before_first_request
+    def clear_lang():
+        session.pop("lang", None)
 
     # Create a user to test with
     @app.before_first_request
@@ -133,17 +153,25 @@ def create_app():
         init_db()
         db_session.commit()
         test_acct = "test@test.com"
-        if not user_datastore.get_user(test_acct):
+        if not user_datastore.find_user(email=test_acct):
             add_user(user_datastore, test_acct, "password", ["admin"])
             print("Created User: {} with password {}".format(test_acct, "password"))
 
+    @app.after_request
+    def allow_absolute_redirect(r):
+        # This is JUST to test odd possible redirects that look relative but are
+        # interpreted by browsers as absolute.
+        # DON'T SET THIS IN YOUR APPLICATION!
+        r.autocorrect_location_header = False
+        return r
+
     @user_registered.connect_via(app)
     def on_user_registered(myapp, user, confirm_token, **extra):
-        flash("To confirm {} - go to /confirm/{}".format(user.email, confirm_token))
+        flash(f"To confirm {user.email} - go to /confirm/{confirm_token}")
 
     @reset_password_instructions_sent.connect_via(app)
     def on_reset(myapp, user, token, **extra):
-        flash("Go to /reset/{}".format(token))
+        flash(f"Go to /reset/{token}")
 
     @tf_security_token_sent.connect_via(app)
     def on_token_sent(myapp, user, token, method, **extra):
@@ -176,6 +204,11 @@ def create_app():
     def basic():
         return render_template_string("Basic auth success")
 
+    @app.route("/protected")
+    @auth_required()
+    def protected():
+        return render_template_string("Protected endpoint")
+
     return app
 
 
@@ -189,9 +222,7 @@ from sqlalchemy import create_engine
 from sqlalchemy.orm import scoped_session, sessionmaker
 from sqlalchemy.ext.declarative import declarative_base
 
-engine = create_engine(
-    "sqlite:////tmp/view_scaffold.db?check_same_thread=False", convert_unicode=True
-)
+engine = create_engine("sqlite:////tmp/view_scaffold.db?check_same_thread=False")
 db_session = scoped_session(
     sessionmaker(autocommit=False, autoflush=False, bind=engine)
 )
@@ -247,6 +278,7 @@ class Role(Base, RoleMixin):
 class User(Base, UserMixin):
     __tablename__ = "user"
     id = Column(Integer, primary_key=True)
+    fs_uniquifier = Column(String(64), unique=True, nullable=False)
     email = Column(String(255), unique=True)
     username = Column(String(255), unique=True, nullable=True)
     password = Column(String(255), nullable=False)
@@ -265,7 +297,7 @@ class User(Base, UserMixin):
     us_totp_secrets = Column(Text(), nullable=True)
     us_phone_number = Column(String(128), nullable=True)
 
-    roles = relationship(
+    roles: t.List = relationship(
         "Role", secondary="roles_users", backref=backref("users", lazy="dynamic")
     )
 
